# encoding: utf-8
require 'rubygems'
require 'active_resource'

# definition du format PDF
module ActiveResource
  module Formats
    module PdfFormat
      extend self

      def initialize(collection_name)
        @collection_name = collection_name.to_s
      end

      def extension
        "pdf"
      end

      def mime_type
        "application/pdf"
      end

      def encode(hash, options = nil)
        nil
      end

      def decode(pdf)
        {
          pdf_body: pdf
        }
      end
    end
  end
end

module Facturation
  class Base < ActiveResource::Base
    # utilisez les variables d'environnement pour configurer l'authentification
    self.site = "https://www.facturation.pro/firms/#{ENV['FACTURATION_FIRM_ID']}"
    self.user = ENV['FACTURATION_API_ID']
    self.password = ENV['FACTURATION_API_KEY']

    self.timeout = 15
    self.format = :json
    self.include_root_in_json = false

    # pour obtenir des logs des requêtes
    # self.logger = Logger.new(STDOUT).tap do |log|
    #   log.level = Logger::DEBUG
    # end

    # methode alternative à l'utilisation des variables d'environement pour vous authentifier
    # ex: faire l'appel suivant dans un fichier d'initialisation
    # Facturation::Base.authenticate_with('API_ID', 'API_PASSWORD', 'FIRM_ID')
    def self.authenticate_with(api_id, api_key, firm_id)
      self.user = api_id
      self.password = api_key
      self.site = "https://wwww.facturation.pro/firms/#{firm_id}"
    end

    class << self
      # Fournir un User-Agent identifiant l'application est recommandé
      # cf. https://www.facturation.pro/api-webservice-facturation
      def headers
        @headers ||= {}
        @headers['User-Agent'] ||= ENV['FACTURATION_APP_NAME'] if ENV['FACTURATION_APP_NAME']
        @headers
      end
    end
  end

  module WithPdf
    # module pour définir l'accès au factures et devis au format PDF
    def self.included(base)
      base.extend(ClassMethods)
    end

    def pdf
      return nil unless id?
      self.class.pdf(id)
    end

    module ClassMethods
      def pdf(id)
        return nil if id.to_i == 0
        current_format, self.format = self.format , :pdf
        res = find(id)
        self.format = current_format
        res.pdf_body
      end
    end
  end

  class Category < Base
  end

  class Item < Base
    # classe utilisée pour les lignes de facturation
    # ne peut pas être utilisée directement pour retrouver un item spécifique ou mettre à jour un item spécifique
  end

  class Settlement < Base
    # Il faut toujours fournir l'identifiant +invoice_id+ de la facture concernée
    # ex: Settlement.all(params: { invoice_id: 1 })
  end

  class Product < Base
  end

  class Supplier < Base
  end

  class Purchase < Base
  end

  class Customer < Base
  end

  class Invoice < Base
    include WithPdf
    if respond_to?(:has_many)
      has_many :items, class_name: 'Facturation::Item'
      has_many :settlements, class_name: 'Facturation::Settlement'
    end

    # pour obtenir le contenu pdf, il suffit d'appeler :
    # Facturation::Invoice.pdf(id)
    # ou bien si la facture est déjà instanciée dans la variable obj, i.e obj = Facturation::Invoice.find(id) :
    # obj.pdf
  end

  class Quote < Base
    include WithPdf
    has_many :items, class_name: 'Facturation::Item'

    # pour obtenir le contenu pdf, il suffit d'appeler
    # Facturation::Quote.pdf(id)
    # ou bien si le devis est déjà instancié dans la variable obj, i.e obj = Facturation::Quote.find(id) :
    # obj.pdf

    # génération d'une facture à partir du devis courant
    # ex: Facturation::Quote.find(id_facture).invoice
    def invoice
      return nil unless id?
      response = post(:invoice)
      result = Facturation::Invoice.new
      result.send(:load_attributes_from_response, response)
      result
    rescue ActiveResource::ResourceInvalid => error
      load_remote_errors(error, true)
      false
    end

    # génération d'une facture à partir du devis +id+
    # ex: Facturation::Quote.invoice(id)
    def self.invoice(id)
      return nil if id.to_i == 0
      find(id).invoice
    end
  end
end
